CREATE TABLE VTLS (
	VTLID INT NOT NULL,
	DEVTYPE INT NOT NULL,
	VTLNAME VARCHAR(40) default NULL,
	VTLTYPE INT default NULL,
	SLOTS INT default NULL,
	IMPEXP INT default NULL,
	DRIVES INT default NULL,
	SERIALNUMBER VARCHAR(32) default NULL,
	PRIMARY KEY (VTLID)
);

CREATE TABLE PHYSSTOR (
	BID SERIAL UNIQUE,
	PID INT default NULL,
	TLID INT default NULL,
	VENDOR VARCHAR(8) default NULL,
	PRODUCT VARCHAR(16) default NULL,
	IDFLAGS BIGINT default NULL,
	T10ID bytea default NULL,
	NAAID bytea default NULL,
	EUI64ID bytea default NULL,
	UNKNOWNID bytea default NULL,
	ISRAID INT default NULL,
	RAIDDEV VARCHAR(512) default NULL,
	PRIMARY KEY (BID)
);

ALTER SEQUENCE physstor_bid_seq MAXVALUE 511 MINVALUE 1 CYCLE;

CREATE TABLE VCARTRIDGE (
	TAPEID SERIAL UNIQUE,
	TLID INT default NULL,
	GROUPID INT default NULL,
	VTYPE INT default NULL,
	WORM INT default 0,
	EADDRESS INT default NULL,
	VSTATUS BIGINT default NULL,
	LABEL VARCHAR(40) default NULL,
	VSIZE VARCHAR(24) default NULL,
	FOREIGN KEY (TLID) REFERENCES VTLS (VTLID) ON DELETE CASCADE
);

ALTER SEQUENCE vcartridge_tapeid_seq MAXVALUE 268435455 MINVALUE 1 CYCLE;

CREATE TABLE VDRIVES (
	TLID INT default NULL,
	TARGETID BIGINT default NULL,
	DRIVETYPE INT default NULL,
	NAME VARCHAR(40) default NULL,
	SERIALNUMBER VARCHAR(32) default NULL,
	FOREIGN KEY (TLID) REFERENCES VTLS (VTLID) ON DELETE CASCADE
);

CREATE TABLE ISCSICONF (
	TLID INT default NULL,
	TARGETID INT default NULL,
	INCOMINGUSER VARCHAR(40) default NULL,
	INCOMINGPASSWD VARCHAR(40) default NULL,
	OUTGOINGUSER VARCHAR(40) default NULL,
	OUTGOINGPASSWD VARCHAR(40) default NULL,
	IQN VARCHAR(256) default NULL,
	FOREIGN KEY (TLID) REFERENCES VTLS (VTLID) ON DELETE CASCADE
);

CREATE TABLE SYSINFO (
	DCHECK INT default 0,
	SYS_RID  VARCHAR(40)
);

CREATE TABLE STORAGEGROUP (
        NAME VARCHAR(40) default NULL,
	WORM INT default 0,
	DEDUPE INT default 1,
	DEDUPEMETA INT default 0,
	AUTOEXPORT INT default 0,
	AUTOREPLICATE INT default 0,
	THRESHOLD INT default 0,
	GROUPID SERIAL UNIQUE
);
ALTER SEQUENCE storagegroup_groupid_seq MAXVALUE 255 MINVALUE 1 CYCLE;

CREATE TABLE FCCONFIG (
	WWPN VARCHAR(40) default NULL,
	TARGETID INT default NULL,
	RULE INT default 0
);

CREATE TABLE PHYSLIB (
	LIBID SERIAL UNIQUE,
	ENABLED INT default 0,
	DEVTYPE INT default 0,
	VENDOR VARCHAR(10) default NULL,
	PRODUCT VARCHAR(16) default NULL,
	IDFLAGS BIGINT default NULL,
	T10ID bytea default NULL,
	NAAID bytea default NULL,
	EUI64ID bytea default NULL,
	UNKNOWNID bytea default NULL
);

ALTER SEQUENCE physlib_libid_seq MAXVALUE 2147483647 CYCLE;

CREATE TABLE PHYSDRIVE (
	LIBID INT default NULL,
	DRIVEID SERIAL UNIQUE,
	VENDOR VARCHAR(10) default NULL,
	PRODUCT VARCHAR(16) default NULL,
	IDFLAGS BIGINT default NULL,
	T10ID bytea default NULL,
	NAAID bytea default NULL,
	EUI64ID bytea default NULL,
	UNKNOWNID bytea default NULL,
	FOREIGN KEY (LIBID) REFERENCES PHYSLIB (LIBID) ON DELETE CASCADE
);

ALTER SEQUENCE physdrive_driveid_seq MAXVALUE 2147483647 CYCLE;
