#!/bin/bash

validatemd5=0
localfile=""
remotefile=""
rcloneconf="/quadstorvtl/etc/rclone.conf"
impexp=""
size=""
md5=""
firstupload=0
firstuploadsleep=60
cleanup=0
cleanupsleep=30
skipmd5check=0
bandwidth="0"

while getopts "l:r:e:m:s:R:b:FC" opt; do
	case $opt in
	l)
		localfile="$OPTARG"
		;;
	r)
		remotefile="$OPTARG"
		;;
	e)
		impexp="$OPTARG"
		;;
	m)
		md5="$OPTARG"
		;;
	s)
		size="$OPTARG"
		;;
	R)
		rcloneconf="$OPTARG"
		;;
	b)	bandwidth="$OPTARG"
		;;
	F)
		firstupload=1
		;;
	C)
		cleanup=1
		;;
	esac
done

# Source rclone.conf to override defaults
if [ -f $rcloneconf ]; then
	. $rcloneconf
fi

dir=`dirname $0`
. $dir/rcloneops.sh

if  [ "$remotename" = "" ]; then
	exit 22 
fi

if [ -f /quadstorvtl/etc/env.conf ]; then
	. /quadstorvtl/etc/env.conf
fi

if [ $cleanup -eq 1 ]; then
	sleep $cleanupsleep
	remotedir=$remotefile
	rclone_expire $remotename $remotedir /quadstorvtl/tmp/cleanup.log
	exit 0
fi

fname=`basename $localfile`
dname=`dirname $localfile`
dname=`basename $dname`
if [ $impexp -eq 1 ]; then
	impexpstr="export"
else
	impexpstr="import"
fi
locallog="/quadstorvtl/tmp/${dname}_${impexpstr}.log"

echo "who  : `whoami`"  >> $locallog 2>&1
echo "date  : `date`"  >> $locallog 2>&1
echo "PATH  : $PATH"  >> $locallog 2>&1
echo "rcloneconf  : $rcloneconf"  >> $locallog 2>&1
echo "localfile  : $localfile"  >> $locallog
echo "remotefile : $remotefile" >> $locallog
echo "impexp     : $impexp"     >> $locallog
echo "md5        : $md5"        >> $locallog
echo "size       : $size"       >> $locallog
echo "firstupload  : $firstupload"       >> $locallog
echo "bandwidth  : $bandwidth"       >> $locallog

if [ "$localfile" = "" -o "$remotefile" = "" -o "$impexp" = "" ]; then
	echo "Insufficient parameters passed"
	exit 22 
fi

if [ "$impexp" = "1" ]; then #export

	#Validate size passed to the one reported by stat
	fsize=`stat -c%s $localfile`
	if [ $? -ne 0 ]; then
		echo "Warn: Cannot stat $localfile" >> $locallog
		exit 2
	fi
	if [ "$size" != "" -a "$fsize" != "$size" ]; then
		echo "Warn: Mismatch in size $size stat $fsize for $localfile" >> $locallog
		exit 9
	fi

	#If validatemd5 is set, then validate md5sum
	if [ "$md5" != "" -a "$validatemd5" = "1" ]; then
		calcmd5=`md5sum $localfile | awk '{print $1}'`
		if [ "$calcmd5" != "$md5" ]; then
			echo "Warn: Mismatch in calculated md5 $calcmd5 and the one passed $md5 for $localfile" >> $locallog
			exit 9
		fi
	fi
else
	mkdir `dirname $localfile`
fi

if [ "$EUID" = "0" -a "$HOME" = "" ]; then
	export HOME="/root"
fi

localdir=`dirname $localfile`
if [ $firstupload = 1 ]; then
	#First chunk for upload
	#Just so that empty files do not serve the purpose later
	echo "firstupload" >> ${localdir}/firstupload
	remotedir=`dirname ${remotefile}`
	echo "rclone_copy $remotename $locallog ${localdir}/firstupload ${remotedir}/firstupload $bandwidth" >> $locallog
	rclone_copy $remotename $impexp $locallog ${localdir}/firstupload ${remotedir}/firstupload $bandwidth 0
	sleep $firstuploadsleep
	touch  ${localdir}/firstuploaddone > /dev/null 2>&1
	errval=$?
	if [ $errval -ne 0 ]; then
		echo "Failed to touch empty file ${localdir}/firstuploaddone" >> $locallog
		exit $errval 
	fi
elif [ "$impexp" = "1" -a "$md5" != "" -a "$firstupload" != "0" ]; then
	iter=0
	while [ $iter -lt 60 ]; do
		if [ -f ${localdir}/firstuploaddone ]; then
			break;
		fi
		sleep 2
		let iter=iter+1
		if [ $iter -eq 60 ]; then
			exit 1
		fi
	done
fi

if [ "$skipmd5check" = "1" -o "$impexp" = "0" ]; then
	md5=""
fi

if [ "$skipmd5check" = "1" -o "$impexp" = "0" -o "$firstupload" = "0" ]; then
	echo "rclone_copy $remotename $locallog $localfile $remotefile $bandwidth" >> $locallog
	rclone_copy $remotename $impexp $locallog $localfile $remotefile $bandwidth 1 $md5
else
	echo "exec $dir/rclonecopy.sh $impexp $rcloneconf $locallog $localfile $remotefile $bandwidth $md5"  >> $locallog
	exec $dir/rclonecopy.sh $impexp $rcloneconf $locallog $localfile $remotefile $bandwidth $md5 >> $locallog 2>&1
	errval=$?
	if [ $errval -ne 0 ]; then
		echo "Error: $cmd failed with error $errval" >> ${locallog}
		exit $errval 
	fi
fi
