#!/bin/bash

if [ ! -f /quadstorvtl/etc/mail.conf ]; then
	exit 1
fi

. /quadstorvtl/etc/mail.conf

if [ "$mailer" = "" -o "$sender" = "" -o "$ccrecipients" != "" ]; then
	exit 1
fi

dedupesize=0

while getopts "p:P:v:V:g:G:m:l:t:s:a:b:R:W:E:T:B:C:I:J:D:" opt; do
	case $opt in
	p)
		label="$OPTARG"
		;;
	P)
		destlabel="$OPTARG"
		;;
	v)
		vtl="$OPTARG"
		;;
	V)
		destvtl="$OPTARG"
		;;
	g)
		pool="$OPTARG"
		;;
	G)
		destpool="$OPTARG"
		;;
	m)
		jobmsg="$OPTARG"
		;;
	l)
		joblogfile="-a $OPTARG"
		;;
	t)
		jobtype="$OPTARG"
		;;
	s)
		jobstatus="$OPTARG"
		;;
	a)
		startdate="$OPTARG"
		;;
	b)
		enddate="$OPTARG"
		;;
	R)
		readsecs="$OPTARG"
		;;
	W)
		writesecs="$OPTARG"
		;;
	E)
		exportsecs="$OPTARG"
		;;
	T)
		jobsecs="$OPTARG"
		;;
	B)
		trfbytes="$OPTARG"
		;;
	D)
		dedupesize="$OPTARG"
		;;
	G)
		writesize="$OPTARG"
		;;
	H)
		readsize="$OPTARG"
		;;
	K)
		exportsize="$OPTARG"
		;;
	C)
		chunks="$OPTARG"
		;;
	I)
		dipaddress="$OPTARG"
		;;
	J)
		sipaddress="$OPTARG"
		;;
	esac
done

subject="${jobtype} job report for ${label}"
msg="VTL: ${vtl}"
if [ "$destvtl" != "" ]; then
	msg="${msg}\nDestination VTL: ${destvtl}"
fi
msg="${msg}\nLabel: ${label}"
if [ "$destlabel" != "" ]; then
	msg="${msg}\nDestination label: ${destlabel}"
fi
msg="${msg}\nPool: ${pool}"
if [ "$destpool" != "" ]; then
	msg="${msg}\nDestination pool: ${destpool}"
fi
if [ "$dipaddress" != "" ]; then
	msg="${msg}\nDestination IP: ${dipaddress}"
fi
if [ "$sipaddress" != "" ]; then
	msg="${msg}\nSource IP: ${sipaddress}"
fi
msg="${msg}\nJob type: ${jobtype}"
msg="${msg}\nStart date: ${startdate}"
msg="${msg}\nEnd date: ${enddate}"
msg="${msg}\nJob time (secs): ${jobsecs}"

if [ $trfbytes -gt 1073741824 ]; then
	trfgb=`awk "BEGIN {print ${trfbytes}/1073741824}"`
	trfgb=`printf %.02f "${trfgb}"`
	msg="${msg}\nTransfer size (GB): ${trfgb}"
else
	trfgb=`awk "BEGIN {print ${trfbytes}/1048576}"`
	trfgb=`printf %.02f "${trfgb}"`
	msg="${msg}\nTransfer size (MB): ${trfgb}"
fi

if [ $dedupesize -gt 1073741824 ]; then
	dedupe=`awk "BEGIN {print ${dedupesize}/1073741824}"`
	dedupe=`printf %.02f "${dedupe}"`
	msg="${msg}\nDedupe size (GB): ${dedupe}"
elif [ $dedupesize -ne 0 ]; then
	dedupe=`awk "BEGIN {print ${dedupesize}/1048576}"`
	dedupe=`printf %.02f "${dedupe}"`
	msg="${msg}\nDedupe size (MB): ${dedupe}"
fi

if [ "$chunks" != "" ]; then
	msg="${msg}\nFile chunks: ${chunks}"
fi

msg="${msg}\nJob status: ${jobstatus}"
if [ "$jobmsg" != "" ]; then
	msg="${msg}\nJob message: ${jobmsg}"
fi

if [ "$ccrecipients" != "" ]; then
	ccrecipients="-c $ccrecipients"
fi

if [ "$bccrecipients" != "" ]; then
	bccrecipients="-b $bccrecipients"
fi

echo -e $msg | ${mailer} -s "${jobstatus}: $subject" -r ${sender} $joblogfile ${ccrecipients} ${bccrecipients} ${recipients}
