#rclone ops
#rcloneflags is sourced from rclone.conf 
#maxdeleteretries is sourced from rclone.conf 
#remotecleanup is sourced from rclone.conf

rclone_expire() {
	_remotename=$1
	_remotedir=$2
	_locallog=$3

	if [ "${_remotename}" = "" -o "${_remotedir}" = "" -o ${_locallog} = "" ]; then
		exit 22 
	fi

	iter=0

	cmd="rclone purge ${_remotename}/${_remotedir}/ $rcloneflags"
	echo "`date` iter $iter $cmd" >> ${_locallog}
	$cmd >> ${_locallog} 2>&1

	errval=$?
	if [ $errval -ne 0 ]; then
		cmd="rclone lsd ${_remotename}/ $rcloneflags | grep -q ${_remotedir}"
		echo "`date` exec $cmd" >> ${_locallog}
		$cmd >> ${_locallog} 2>&1
		greperrval=$?
		if [ $greperrval -ne 0 ]; then
			echo "Error: $cmd failed with error $errval $greperrval" >> ${_locallog}
			exit $errval
		else
			exit 0
		fi
	else
		cmd="rclone rmdir ${_remotename}/${_remotedir} $rcloneflags"
		echo "`date` iter $iter $cmd" >> ${_locallog}
		$cmd >> ${_locallog} 2>&1
	fi

	if [ "$remotecleanup" != "0" ]; then
		echo "`date` rclone cleanup ${remotename} $rcloneflags" >> ${_locallog}
		rclone cleanup ${remotename} $rcloneflags  >> ${_locallog} 2>&1
	fi
}

rclone_copy() {
	_remotename=$1
	_impexp=$2
	_locallog=$3
	_localfile=$4
	_remotefile=$5
	_bandwidth=$6
	_doexec=$7
	_md5=$8
	_remotedir=`dirname ${_remotefile}`
	_localdir=`dirname ${_localfile}`

	echo "remotename: ${_remotename} impexp ${_impexp} localog ${_locallog} localfile  ${_localfile} remotefile ${_remotefile} doexec ${_doexec}" >> ${_locallog}
	if [ "${_remotename}" = "" -o "${_remotedir}" = "" -o ${_locallog} = "" ]; then
		exit 22 
	fi

	if [ "${_impexp}" = "1" ]; then
		cmd="rclone copy ${_localfile} ${_remotename}/${_remotedir}/ $rcloneflags"
	else
		mkdir -p ${_localdir}
		cmd="rclone copy ${_remotename}/${_remotefile} ${_localdir} $rcloneflags"
	fi

	if [ "${_bandwidth}" != "" -a "${_bandwidth}" != "0" ]; then
		cmd="$cmd --bwlimit ${_bandwidth}"
	fi

	echo "$cmd" >> ${_locallog}
	if [ ${_doexec} -eq 1 ]; then
		exec $cmd >> ${_locallog} 2>&1
	else
		rclone mkdir ${_remotename}/${_remotedir}/
		echo "rclone mkdir ${_remotename}/${_remotedir}/"  >> ${_locallog}
		iter=0
		while [ $iter -lt 20 ]; do
			sleep 5
			rclone ls ${_remotename}/${_remotedir}/ > /dev/null 2>&1
			errval=$?
			if [ $errval -eq 0 ]; then
				break
			elif [ $errval -ne 3 ]; then
				echo "Error: rclone ls ${_remotename}/${_remotedir}/  error $errval" >> ${_locallog}
				exit $errval 
			fi
			let iter=iter+1
		done
		echo "Done create remote directory  ${_remotename}/${_remotedir}/ slept $iter times" >> ${_locallog}
		$cmd >> ${_locallog} 2>&1
	fi

	errval=$?
	if [ $errval -ne 0 ]; then
		echo "Error: $cmd failed with error $errval" >> ${_locallog}
		if [ "${_impexp}" = "1" ]; then
			rclone delete ${_remotename}/${_remotefile} $rcloneflags >> ${_locallog} 2>&1
		fi
		exit $errval 
	fi

	if [ "${_md5}" = "" ]; then
		return
	fi

	cmd="rclone md5sum ${_remotename}/${_remotefile} $rcloneflags"
	newsum=`$cmd | awk '{print $1}'`
	errval=$?
	if [ $errval -ne 0 ]; then
		echo "Error: $cmd failed with error $errval" >> ${_locallog}
		if [ "${_impexp}" = "1" ]; then
			rclone delete ${_remotename}/${_remotefile} $rcloneflags >> ${_locallog} 2>&1
		fi
		exit $errval 
	fi

	if [ "$newsum" != "${_md5}" ]; then
		echo "Error: Mismatch in md5sum for ${_remotename}/${_remotefile} expected :${_md5}: got :${newsum}:" >> ${_locallog}
		output=`$cmd`
		echo "for $cmd output $output" >> ${_locallog}
		if [ "${_impexp}" = "1" ]; then
			rclone delete ${_remotename}/${_remotefile} $rcloneflags >> ${_locallog} 2>&1
		fi
		exit 126
	fi

	echo "Verified md5sum for ${_remotename}/${_remotefile} expected ${_md5} got ${newsum}" >> ${_locallog}
}
