#!/bin/bash

check_error() {
	if [ "$?" == "0" ]; then
		return
	fi
	echo "$1"
	exit 1
}

tables="sysinfo"

#comment below to skip vtl configuration dump 
tables="$tables vtls vdrives iscsiconf"

#comment below to skip vtl inquiry
tables="$tables inquiry"

#comment below to skip storage pool, disk and vcartridge configuration dump
tables="$tables storagegroup physstor vcartridge"

#comment below to skip FC access configuration restore
tables="$tables fcconfig fchost fcport"

#comment below to skip file export job history
tables="$tables expjobs"

#comment below to skip export schedule
tables="$tables fschedule eschedule bschedule"

#comment below to skip replication job history
tables="$tables repjobs"

#comment below to skip tape job history
tables="$tables tapejobs"

#comment below to skip physical library configuration
tables="$tables physlib physdrive"

#comment below to skip user authentication
tables="$tables autht"

#comment below to skip iscsi chap configuration
tables="$tables iscsichap"

mkdir -p /tmp/vtdump/
if [ -d /quadstorvtl/pgsqlsys ]; then
	chown postgres /tmp/vtdump/
else
	chown vtdbuser /tmp/vtdump/
fi
rm -f /tmp/vtdump/* > /dev/null 2>&1

/quadstorvtl/pgsql/etc/pgsql start > /dev/null 2>&1
check_error "Cannot start database"
 
for table in $tables; do
	echo "Dumping table $table"
	if [ -d /quadstorvtl/pgsqlsys ]; then
		pg_dump -p 9985 -U postgres -a --column-inserts -t $table qsdb > /tmp/vtdump/$table.sql
	else
		/quadstorvtl/pgsql/bin/pg_dump -U vtdbuser -a --column-inserts -t $table qsdb > /tmp/vtdump/$table.sql
	fi
	check_error "Cannot dump table $table"
done
unlink /tmp/vtdump.tgz > /dev/null 2>&1

cd /tmp && tar czf vtdump.tgz vtdump/ > /dev/null 2>&1

#Cleanup again
rm -f /tmp/vtdump/* > /dev/null 2>&1
rmdir /tmp/vtdump > /dev/null 2>&1
