#!/bin/bash
if [ $# -lt 1 ]; then
	echo "ERROR: Invalid number of arguments $#"
	echo "Usage: sqldumpfilejobs.sh <output file>"
	exit 1
fi

filename=$1
fname=`echo ${filename%.*}`
logger "Dumping expjobs table to ${fname}"

unlink ${fname} > /dev/null 2>&1

if [ -d /quadstorvtl/pgsqlsys ]; then
	pg_dump -p 9985 -U postgres -a --column-inserts -t expjobs qsdb > ${fname}
else
	/quadstorvtl/pgsql/bin/pg_dump -U vtdbuser -a --column-inserts -t expjobs qsdb > ${fname}
fi

if [ "$?" != "0" ]; then
	logger "Failed to dump expjobs table to ${fname}"
	unlink ${fname}
	exit 1
fi

logger "Unlinking older ${1}"
unlink $1 > /dev/null 2>&1

cat ${fname} | sed -e "s:expjobs (expid, :expjobs (:g" | sed -E 's/VALUES [(][0-9]+,/VALUES (/g' | grep -v expjobs_expid_seq  > ${fname}.tmp
mv -f ${fname}.tmp ${fname}

logger "gzipping ${fname}"
gzip ${fname}

stat $1 > /dev/null 2>&1

if [ "$?" != "0" ]; then
	logger "Cannot stat $1"
	unlink ${fname}
	exit 1
fi

exit 0
