#!/bin/bash
check_error() {
	if [ "$?" == "0" ]; then
		return
	fi
	echo "$1"
	exit 1
}

if [ $# -ne 1 ]; then
	echo "ERROR: Invalid number of arguments $#"
	echo "Usage: sqlrestore.sh <dump tar file>"
	exit 1
fi

tarfile=$1

rm -f /tmp/vtdump/* > /dev/null 2>&1
rmdir /tmp/vtdump > /dev/null 2>&1

/quadstorvtl/pgsqlsys/etc/pgsql start > /dev/null 2>&1
check_error "Cannot start database"

cd /tmp && tar xzf $tarfile

tables="sysinfo vtls vdrives iscsiconf inquiry storagegroup physstor vcartridge fcconfig fchost fcport expjobs fschedule eschedule bschedule repjobs tapejobs physlib physdrive autht iscsichap"

#Cleanup table data
for table in $tables; do
	if [ ! -f /tmp/vtdump/${table}.sql ]; then
		echo "Skipping table $table"
		continue
	fi
	echo "Deleting table $table"
	if [ -d /quadstorvtl/pgsqlsys ]; then
		echo "delete from $table *" | psql -p 9985 -U postgres qsdb > /dev/null 2>&1
	else
		echo "delete from $table *" | /quadstorvtl/pgsql/bin/psql -U vtdbuser qsdb > /dev/null 2>&1
	fi
done

for table in $tables; do
	if [ ! -f /tmp/vtdump/${table}.sql ]; then
		continue
	fi
	echo "Restoring table $table"
	if [ -d /quadstorvtl/pgsqlsys ]; then
		psql -p 9985 -U postgres -f /tmp/vtdump/${table}.sql qsdb > /dev/null 2>&1
	else
		/quadstorvtl/pgsql/bin/psql -U vtdbuser -f /tmp/vtdump/${table}.sql qsdb > /dev/null 2>&1
	fi
done

#Cleanup again
rm -f /tmp/vtdump/* > /dev/null 2>&1
rmdir /tmp/vtdump > /dev/null 2>&1

