/* 
 * Copyright (C) Shivaram Upadhyayula <shivaram.u@quadstor.com>
 */

#ifndef QS_COMMONDEFS_H_
#define QS_COMMONDEFS_H_ 1

#include "compilerdefs.h"
#include "drivedefs.h"

#define QUADSTOR_VTL_VERSION   "3.0.79.26"

#define TL_MAX_DISKS	512
#define TL_MAX_POOLS	512	
#define TL_DEV_NAME	"vtiodev"
#ifdef ENABLE_FCINT
#define FC_DEV_NAME	"qlascdev"
#define FC_DEV		"/dev/qlascdev"
#else
#define FC_DEV_NAME	"ql2xapidev"
#define FC_DEV		"/dev/ql2xapidev"
#endif
#define TL_DEV		"/dev/vtiodev"
#define MIN_PHYSDISK_SIZE	(1ULL << 32)
#define MDAEMON_USR_PATH	"/var/run/vtmdaemonusr.sock"
#define CDAEMON_RECV_PATH	"/var/run/vtcdaemonrcv.sock"


/* Limits */

#define DELETE_MAGIC			0x5a4A3a2A
#define TL_RID_MAX	40
struct mdaemon_info {
	int disable_dedupe;
	int disable_compression;
	int disable_vault;
	int cmd_debug;
	int access_debug;
	int extended_logging;
	int reservation_debug;
	int scan_debug;
	int dedupe_threshold;
	int rcache_threshold;
	int hcache_threshold;
	int hcache_bits;
	int hcache_tapes;
	int disable_hcache;
	int enable_hcache;
	int amap_threshold;
	int index_threshold;
	int ext_dedupe_mem;
	int flush_dedupe_mem;
	int delete_threads;
	int sequential_tables;
	int debug_tape_position;
	int debug_load;
	int debug_meta;
	int debug_unmap;
	int skip_free_space_check;
	int retry_uncomp_err;
	int ft_flags;
	int emulate_ib;
	int emulate_hh;
	int report_wwpn;
	int emulate_oem;
	int map_threshold;
	int map_threshold_v2;
	int map_threshold_v3;
	int wlist_threshold;
	int wlist_threshold_min;
	int rep_sync_block;
	int rep_sync_sock;
	int multi_init;
	int disable_hash_spanning;
	int write_protect_import;
	int tdrive_buffered_mode;
	int tdrive_write_cache_size;
	uint32_t low_space_threshold;
	uint32_t low_space_delay;
	uint32_t ddtable_sync_low;
	uint32_t ddtable_sync_high;
	uint32_t ddtable_sync_pause;
	uint32_t ddtable_sync_debug;
	int v5_tape;
	int enable_encryption;
	int peer_count;
	int disable_uncomp_v2;
	int meta_load_timeout;
	int meta_load_retries;
	int migration_threads;
	int sock_read_timeout;
	uint32_t group_threshold_check_start;
	uint32_t group_threshold_check_step;
	uint32_t group_threshold_check_interval;
	unsigned long ew_size;
	unsigned long rep_sync_size;
};

enum {
	FT_REPLICATION 		= 0x1,
	FT_TAPE_EXPORT 		= 0x2,
	FT_FILE_EXPORT 		= 0x4,
	FT_MULTI_PARTITION	= 0x8,
	FT_MULTI_LIB		= 0x10,
	FT_MULTI_DISK		= 0x20,
};

static inline int
ft_enabled(struct mdaemon_info *mdaemon_info, int id)
{
	return (mdaemon_info->ft_flags & id);
}

static inline int
ft_replication(struct mdaemon_info *mdaemon_info)
{
	return ft_enabled(mdaemon_info, FT_REPLICATION);
}

static inline int
ft_tape_export(struct mdaemon_info *mdaemon_info)
{
	return ft_enabled(mdaemon_info, FT_TAPE_EXPORT);
}

static inline int
ft_multi_partition(struct mdaemon_info *mdaemon_info)
{
	return ft_enabled(mdaemon_info, FT_MULTI_PARTITION);
}

static inline int
ft_file_export(struct mdaemon_info *mdaemon_info)
{
	return ft_enabled(mdaemon_info, FT_FILE_EXPORT);
}

static inline int
ft_multi_lib(struct mdaemon_info *mdaemon_info)
{
	return ft_enabled(mdaemon_info, FT_MULTI_LIB);
}

static inline int
ft_multi_disk(struct mdaemon_info *mdaemon_info)
{
	return ft_enabled(mdaemon_info, FT_MULTI_DISK);
}

#define UMA_INFO_BUFFER_SIZE	65536
struct uma_info {
	char name[32];
	uint64_t objs;
	uint64_t alloced;
	uint32_t umabufs;
	uint32_t free_umabufs;
	uint32_t size;
	uint32_t objs_per_umabuf;
	uint32_t umabuf_size;
};

struct uma_spec {
	uint8_t *buffer;
};

struct map_slot {
	uint32_t libid;
	int tl_id;
	int unmap;
	char label[40];
	char errmsg[256];
};

PACKED(struct raw_export_state {
	uint64_t export_lid;
	uint32_t volume_change_reference;
});

#define TDISK_NAME_LEN		36
#define TL_NAME_LEN		36
#define TL_MAX_NAME_LEN		40
#define TL_MAX_POOLDIR_LEN	1024
#define TDISK_MAX_NAME_LEN	40
#define MAX_TAPE_PARTITIONS	4

struct vcartridge {
	char label[40];
	char newlabel[40];
	char desc[128];
	char group_name[TDISK_MAX_NAME_LEN];
	char sys_rid[TL_RID_MAX];
	uint64_t size;
	uint64_t used; /* use percentage */
	uint64_t data_size;
	uint32_t vstatus;
	uint32_t group_id;
	int tl_id;
	uint32_t tape_id;
	uint8_t worm; /* worm support */
	uint8_t ltfs;
	uint8_t use_free_slot;
	uint8_t use_free_drive;
	uint8_t update_vtl;
	uint8_t type;
	uint8_t elem_type;
	uint8_t modify_label;
	uint16_t elem_address;
	uint8_t free_alloc;
	uint8_t loaderror;
	uint8_t file;
	uint8_t busy;
	uint8_t vault;
	uint8_t erase;
	uint8_t import;
	uint8_t enable_encryption;
	struct group_info *group;
	uint64_t export_date;
	uint64_t last_write;
	uint64_t replication_date;
	uint64_t media_id;
	uint8_t  medium_serial_number[32];
	struct raw_export_state export_states[MAX_TAPE_PARTITIONS];
	TAILQ_ENTRY(vcartridge) g_entry;
	TAILQ_ENTRY(vcartridge) q_entry;
};

struct tdrive_stats {
	uint8_t  compression_enabled;
	uint32_t read_errors;
	uint32_t write_errors;
	uint32_t load_count;
	uint32_t locate_cmds;
	uint32_t space_cmds;
	uint32_t rewind_cmds;
	uint64_t write_ticks;
	uint64_t read_ticks;
	uint64_t pending_write_ticks;
	uint64_t pending_write_cnt;
	uint64_t buffered_write_cnt;
	uint64_t write_bytes_processed;
	uint64_t read_bytes_processed;
	uint64_t bytes_read_from_tape;
	uint64_t bytes_written_to_tape;
	uint64_t compressed_bytes_read;
	uint64_t compressed_bytes_written; 
	uint64_t write_bytes_processed_mount;
	uint64_t read_bytes_processed_mount;
	uint64_t bytes_read_from_tape_mount;
	uint64_t bytes_written_to_tape_mount;
	uint64_t dedupe_blocks_mount;
	uint64_t write_flush_mount;
	uint64_t write_flush;
	uint64_t map_cnt_mount;
	uint64_t map_cnt;
	uint64_t zero_blocks;
	uint64_t dedupe_blocks;
	uint64_t dedupe_skipped;
	uint64_t scan_skip;
	uint64_t skip_till_dend;
	uint64_t verify_hits;
	uint64_t verify_misses;
	uint64_t verify_ticks_mount;
	uint64_t verify_ticks;
	uint64_t ili_block_size;
	uint64_t write_cmds;
	uint64_t read_cmds;
	uint64_t read_cmd_ticks;
	uint64_t read_kernel_ticks;
	uint64_t read_usr_ticks;
	uint64_t read_interface_ticks;
	uint64_t read_dxfer_ticks;
	uint64_t write_cmd_ticks;
	uint64_t write_kernel_ticks;
	uint64_t write_usr_ticks;
	uint64_t write_interface_ticks;
	uint64_t write_allocate_buffers_ticks;
	uint64_t write_dxfer_ticks;
};

struct libctl {
	uint32_t tl_id;
	uint16_t element_address;
	uint16_t real_address;
	uint8_t element_type;
	uint8_t access;
	uint8_t access_set;
	uint8_t except_flags;
	uint8_t except_set;
	uint8_t unload;
	uint8_t unload_set;
	uint8_t set;
	char label[40];
	char errmsg[128];
};

struct devicectl {
	/* Device ctl stuff */
	uint8_t offline;
	uint8_t offline_response_code;
	uint8_t offline_sense_key;
	uint8_t offline_asc;
	uint8_t offline_ascq;
	uint8_t report_naa;
	uint8_t report_eui;
	uint8_t report_wwn;
	uint8_t ssc_version;
	uint8_t spc_version;
	uint8_t smc_version;
	uint8_t enable_smc3;
	uint8_t enable_ssc4;
};

struct vdeviceinfo {
	int tl_id;
	int iscsi_tid;
	int vhba_id;
	int lun_id;
	int slots;
	int ieports;
	int drives;
	int dtstart;
	int ststart;
	int iestart;
	int make;
	int vtype;
	int disdedupe;
	int compression;
	int seriallen;
	int inquirylen;
	int avoltag;
	uint32_t dedupe_block_size;
	char name[40];
	int autoreplicate;
	int type;
	int librarymode;
	uint8_t tape_loaded;
	uint8_t free_alloc;
	uint8_t enable_compression;
	uint8_t mod_type; /* modification type */
	uint8_t mapped_drive;
	uint8_t tape_label[40];
	uint32_t tape_id;
	uint32_t target_id;
	char serialnumber[40];
	char vendor[16];
	char product[40];
	char revision[8];
	char prefix[40];
	char suffix[40];
	char sys_rid[TL_RID_MAX];
	char inquiry_buf[255];
	unsigned long rand1;
	unsigned long rand2;
	struct tdrive_stats stats;

	struct devicectl ctl;
};

#define DRIVE_MOD_TYPE_NONE		0x00
#define DRIVE_MOD_TYPE_COMPRESSION	0x01

#define TL_MAX_DEVICES		1024
#define TL_MAX_DEVICES_INT	2048
#define TL_DEVICES_PER_BUS	8
#define TL_MAX_DRIVES		64
#define TL_MAX_DRIVES_LEGACY	7
#define MAX_DINFO_DEVICES	65537

static inline int
slottype_from_drivetype(int drivetype)
{
	switch (drivetype)
	{
		case DRIVE_TYPE_VHP_DLTVS80:
			return VOL_TYPE_DLT_4;
		case DRIVE_TYPE_VHP_DLTVS160:
			return VOL_TYPE_VSTAPE;
		case DRIVE_TYPE_VHP_SDLT220:
			return VOL_TYPE_SDLT_1;
		case DRIVE_TYPE_VHP_SDLT320:
			return VOL_TYPE_SDLT_2;
		case DRIVE_TYPE_VHP_SDLT600:
			return VOL_TYPE_SDLT_3;
		case DRIVE_TYPE_VHP_ULT232:
			return VOL_TYPE_LTO_1;
		case DRIVE_TYPE_VHP_ULT448:
			return VOL_TYPE_LTO_2;
		case DRIVE_TYPE_VHP_ULT460:
			return VOL_TYPE_LTO_2;
		case DRIVE_TYPE_VHP_ULT960:
			return VOL_TYPE_LTO_3;
		case DRIVE_TYPE_VHP_ULT1840:
			return VOL_TYPE_LTO_4;
		case DRIVE_TYPE_VHP_ULT3280:
			return VOL_TYPE_LTO_5;
		case DRIVE_TYPE_VHP_ULT6250:
			return VOL_TYPE_LTO_6;
		case DRIVE_TYPE_VHP_ULT15000:
			return VOL_TYPE_LTO_7;
		case DRIVE_TYPE_VHP_ULT30750:
			return VOL_TYPE_LTO_8;
		case DRIVE_TYPE_VHP_ULT45000:
			return VOL_TYPE_LTO_9;
		case DRIVE_TYPE_VIBM_3580ULT1:
			return VOL_TYPE_LTO_1;
		case DRIVE_TYPE_VIBM_3580ULT2:
			return VOL_TYPE_LTO_2;
		case DRIVE_TYPE_VIBM_3580ULT3:
			return VOL_TYPE_LTO_3;
		case DRIVE_TYPE_VIBM_3580ULT4:
			return VOL_TYPE_LTO_4;
		case DRIVE_TYPE_VIBM_3580ULT5:
			return VOL_TYPE_LTO_5;
		case DRIVE_TYPE_VIBM_3580ULT6:
			return VOL_TYPE_LTO_6;
		case DRIVE_TYPE_VIBM_3580ULT7:
			return VOL_TYPE_LTO_7;
		case DRIVE_TYPE_VIBM_3580ULT8:
			return VOL_TYPE_LTO_8;
		case DRIVE_TYPE_VIBM_3580ULT9:
			return VOL_TYPE_LTO_9;
		case DRIVE_TYPE_VQUANTUM_SDLT220:
			return VOL_TYPE_SDLT_1; 
		case DRIVE_TYPE_VQUANTUM_SDLT320:
			return VOL_TYPE_SDLT_2; 
		case DRIVE_TYPE_VQUANTUM_SDLT600:
			return VOL_TYPE_SDLT_3;
		case DRIVE_TYPE_VSTK_T9840A:
			return VOL_TYPE_T9840_A;
		case DRIVE_TYPE_VSTK_T9840B:
			return VOL_TYPE_T9840_B;
		case DRIVE_TYPE_VSTK_T9840C:
			return VOL_TYPE_T9840_C;
		case DRIVE_TYPE_VSTK_T9840D:
			return VOL_TYPE_T9840_D;
		case DRIVE_TYPE_VSTK_T9940A:
			return VOL_TYPE_T9940_A;
		case DRIVE_TYPE_VSTK_T9940B:
			return VOL_TYPE_T9940_B;
		case DRIVE_TYPE_VSTK_T10000A:
			return VOL_TYPE_T10000_A;
		case DRIVE_TYPE_VSTK_T10000B:
			return VOL_TYPE_T10000_B;
		case DRIVE_TYPE_VSTK_T10000C:
			return VOL_TYPE_T10000_C;
		case DRIVE_TYPE_VSTK_T10000D:
			return VOL_TYPE_T10000_D;
	}
	return -1;
}

struct bint_stats {
	uint64_t dedupe_blocks;
	uint64_t compressed_size;
	uint64_t uncompressed_size;
	uint64_t total_blocks;
	uint64_t compression_hits;
	uint64_t zero_blocks;
	uint64_t pad2;
	uint64_t pad3;
	uint64_t pad4;
	uint64_t pad5;
	uint64_t pad6;
	uint64_t pad7;
	uint64_t pad8;
	uint64_t pad9;
	uint64_t pad10;
};

#define V2_DISK		0x1
#define RID_SET		0x4

PACKED(struct raw_bdevint {
	uint8_t  ddmaster;
	uint8_t ddbits;
	uint8_t enable_comp;
	int8_t initialized;
	uint8_t flags;
	uint32_t bid;
	uint32_t sector_shift;
	uint64_t usize;
	uint64_t free;
	uint64_t b_start;
	uint64_t b_end;
	uint8_t magic[8];
	uint8_t vendor[8];
	uint8_t product[16];
	uint8_t serialnumber[32];
	uint32_t group_id;
	int32_t group_flags;
	uint8_t write_cache;
	uint8_t quad_prod[4];
	uint8_t dismagic[3];
	uint16_t node_block_bits;
	uint16_t pad2;
	uint32_t pad3;
	uint64_t pad4;
	uint64_t pad5;
	uint64_t pad6;
	uint64_t pad7;
	uint64_t pad8;
	uint64_t pad9;
	uint64_t pad10;
	struct bint_stats stats;
	char mrid[TL_RID_MAX];
	char group_name[TDISK_MAX_NAME_LEN];
	uint8_t ext_serialnumber[224];
});

static inline int
raw_bint_serial_match(struct raw_bdevint *raw_bint, char *serialnumber, int serial_len)
{
	int serial_max, min_len;

	min_len = serial_max = sizeof(raw_bint->serialnumber);
	if (min_len > serial_len)
		min_len = serial_len;

	if (memcmp(raw_bint->serialnumber, serialnumber, min_len))
		return 0;

	if (serial_len > serial_max) {
		if (memcmp(raw_bint->ext_serialnumber, serialnumber + serial_max, serial_len - serial_max))
			return 0;
	}
	return 1;
}

struct group_conf {
	char name[TDISK_MAX_NAME_LEN];
	char pooldir[TL_MAX_POOLDIR_LEN];
	uint32_t group_id;
	int worm;
	int file;
	int dedupe;
	int dedupemeta;
	int autoexport;
	int dismountexport;
	int disableuncompv2;
	int autoreplicate;
	int autodelete;
	int threshold;
	int enable_verify;
	int dedupe_block_size;
	int replace_hash;
};

#define DEFAULT_GROUP_NAME	"Default"

struct bdev_info {
	uint32_t bid;
	char devpath[4096];
	uint64_t size;
	uint64_t usize;
	uint64_t free;
	uint64_t reserved;
	uint8_t  isnew;
	uint8_t  reopen;
	uint8_t  ddmaster;
	uint8_t  ismaster;
	uint8_t  unmap;
	uint8_t  write_cache;
	uint8_t load_post;
	uint8_t first_load;
	int8_t initialized;
	uint8_t disabled;
	uint8_t enable_comp;
	uint8_t  vendor[8];
	uint8_t  product[16];
	uint8_t  serialnumber[256];
	uint32_t free_alloc;
	uint32_t replace_bid;
	int serial_len;
	int use_diskid;
	uint32_t max_index_groups;
	uint32_t group_id;
	struct bint_stats stats;
	uint64_t max_dedupe_mem;
	uint64_t cur_dedupe_mem;
	char rid[TL_RID_MAX];
	char errmsg[256];
};

struct tl_entryinfo {
	uint32_t tl_id;
	uint32_t count;
	uint32_t size;
	uint32_t flags;
	uint32_t error;
};

struct ieinfo_common {
	uint32_t tape_id;
	uint32_t drive_tl_id;
	uint32_t drive_lun_id;
	uint32_t tl_id;
	uint32_t dedupe_block_size;
	int disdedupe;
	int op; /* Map, Unmap etc */
	int jobtype; /* Export or Import */
	int element_address;
	int drive_host_id;
	int drive_type;
	int tape_type;
	char sys_rid[TL_RID_MAX];
	char drive_serialnumber[64];
	char errmsg[256];
};

struct job_stats {
	uint64_t start_time;
	uint64_t queued_time;
	uint64_t end_time;
	uint64_t elapsed_msecs;
	uint64_t write_msecs;
	uint32_t dest_ipaddr;
	uint32_t src_ipaddr;
	uint64_t deduped_blocks;
	uint64_t total_bytes;
	uint64_t encrypted_bytes;
	uint64_t bytes_written;
	uint64_t tblocks;
	uint32_t min_block_size;
	uint32_t max_block_size;
	uint32_t fmarks;
};

struct usr_msg {
	int msg_id;
	int msg_rsp;
	int iscsi_tid;
	int vhba_id;
	uint32_t target_id;
	uint32_t tl_id;
	uint32_t mirror_ipaddr;
	uint32_t pad;
};

struct mapped_move {
	struct usr_msg msg;
	char label[40];
	char device[512];
	uint8_t load;
	uint8_t error_code;
	uint8_t sense_key;
	uint8_t asc;
	uint8_t ascq;
	uint16_t maddress;
	uint32_t libid;
	uint32_t driveid;
	uint32_t host;
	uint32_t channel;
	uint32_t target;
	uint32_t lun;
};

struct vcart_update {
	struct usr_msg msg;
	uint64_t used; /* use percentage */
	uint64_t data_size;
	uint32_t vstatus;
	uint8_t busy;
	uint8_t elem_type;
	uint16_t elem_address;
};

struct usr_job {
	struct usr_msg msg;
	uint64_t job_id;
	struct job_stats stats;
};

struct usr_notify {
	struct usr_msg msg;
	int notify_type;
	int pad;
	char notify_msg[160];
};

enum {
	LIBRARY_MODE_DEFAULT,
	LIBRARY_MODE_SEQUENTIAL,
};

enum {
	USR_NOTIFY_INFO,
	USR_NOTIFY_WARN,
	USR_NOTIFY_ERR,
	USR_NOTIFY_VDISK_THRESHOLD,
	USR_NOTIFY_BINT_CREATE_DONE,
	USR_NOTIFY_WARN_EVT,
};

enum {
	USR_RSP_OK,
	USR_RSP_ERR,
	USR_RSP_BUSY,
};

enum {
	USR_MSG_VCARTRIDGE_DELETED,
	USR_MSG_JOB_COMPLETED,
	USR_MSG_NOTIFY,
	USR_MSG_THRESHOLD_REACHED,
	USR_MSG_VCARTRIDGE_EXPORT_CHECK,
	USR_MSG_VCARTRIDGE_UNMOUNT_CHECK,
	USR_MSG_VCARTRIDGE_EXPORT_CANCEL,
	USR_MSG_VCARTRIDGE_UPDATE,
	USR_MSG_BINT_DELETED,
	USR_MSG_BINT_MASTER_CHANGED,
	USR_MSG_BINT_MIGRATE_COMPLETE,
	USR_MSG_VCARTRIDGE_UNMOUNT_EXPORT_CHECK,
	USR_MSG_MAPPED_MOVE,
	USR_MSG_CLONE_COMPLETED,
	USR_MSG_DDTABLE_FILL_COMPLETE,
};

struct fc_rule_config {
	uint64_t wwpn[2];
	uint64_t target_wwpn;
	uint32_t target_id;
	uint32_t drive_id;
	uint64_t old_wwpn;
	int rule;
};

enum {
	FC_RULE_ALLOW,
	FC_RULE_DISALLOW,
};

enum {
	MIRROR_STATUS_SUCCESSFUL = 0x1,
	MIRROR_STATUS_ERROR,
	MIRROR_STATUS_CANCELED,
	MIRROR_STATUS_INPROGRESS,
};

enum {
	CLONE_STATUS_SUCCESSFUL = 0x1,
	CLONE_STATUS_ERROR,
	CLONE_STATUS_CANCELED,
	CLONE_STATUS_INPROGRESS,
};

struct clone_config {
	uint32_t src_tl_id;
	uint32_t src_tape_id;
	uint32_t dest_tl_id;
	uint32_t dest_tape_id;
	uint32_t dest_ipaddr;
	uint32_t src_ipaddr;
	uint64_t job_id;
	int status;
	int swap;
	char src_vcart[TL_MAX_NAME_LEN];
	char src_group[TL_MAX_NAME_LEN];
	char dest_vcart[TL_MAX_NAME_LEN];
	char dest_group[TL_MAX_NAME_LEN];
	char sys_rid[40];
	char dest_tape_rid[40];
	char errmsg[256];
	struct job_stats stats;
};

struct mirror_config {
	uint32_t src_tl_id;
	uint32_t src_tape_id;
	uint32_t dest_tl_id;
	uint32_t dest_tape_id;
	uint32_t dest_ipaddr;
	uint32_t src_ipaddr;
	uint64_t job_id;
	uint8_t  status;
	char mirror_vcart[TDISK_MAX_NAME_LEN];
	char mirror_group[TDISK_MAX_NAME_LEN];
	char sys_rid[40];
	char dest_tape_rid[40];
	char errmsg[256];
	struct job_stats stats;
};

#define RCACHE_THRESHOLD_MIN			1
#define RCACHE_THRESHOLD_DEF			2
#define RCACHE_THRESHOLD_MAX			6	
#define HCACHE_THRESHOLD_MIN			1
#define HCACHE_THRESHOLD_DEF			5
#define HCACHE_THRESHOLD_MAX			8	
#define INDEX_THRESHOLD_MIN			4
#define INDEX_THRESHOLD_DEFAULT			12
#define INDEX_THRESHOLD_EXT			8	
#define INDEX_THRESHOLD_MAX			60	
#define DEDUPE_THRESHOLD_MIN			56
#define DEDUPE_THRESHOLD_MAX			75

#define MIRROR_RECV_TIMEOUT_MAX			1500
#define MIRROR_CONNECT_TIMEOUT_MAX		300
#define MIRROR_SEND_TIMEOUT_MAX			1500
#define MIRROR_FLUSH_TIMEOUT_MAX		1500

#define MIRROR_RECV_TIMEOUT_MIN			20
#define MIRROR_CONNECT_TIMEOUT_MIN		5
#define MIRROR_SEND_TIMEOUT_MIN			20
#define MIRROR_FLUSH_TIMEOUT_MIN		60

enum {
	WRITE_CACHE_DEFAULT,
	WRITE_CACHE_FLUSH,
	WRITE_CACHE_FUA,
};

enum {
	GROUP_FLAGS_MASTER,
	GROUP_FLAGS_DEDUPEMETA,
	GROUP_FLAGS_WORM,
	GROUP_FLAGS_UNMAP_ENABLED,
	GROUP_FLAGS_UNMAP,
	GROUP_FLAGS_TAIL_META,
	GROUP_FLAGS_DEDUPE,
	GROUP_FLAGS_SKIP_ALLOC_LIST,
	GROUP_FLAGS_STATS_BLOCK,
	GROUP_FLAGS_VARDEDUPE,
	GROUP_FLAGS_DEDUPE_DISABLED,
	GROUP_FLAGS_DEDUPE_DISABLED_ND,
	GROUP_FLAGS_OLD_MASTER,
	GROUP_FLAGS_UNCOMP_V2,
	GROUP_FLAGS_CRC32,
};

enum {
	DRIVE_MOD_WORM		= 0x01,
};

enum {
	MOD_OP_ENABLE		= 0x01,
	MOD_OP_DISABLE		= 0x02,
};

PACKED(struct drive_info_v2 {
	uint16_t target_id;
	uint16_t make;
	char serialnumber[16];
});

PACKED(struct drive_info {
	uint16_t target_id;
	uint16_t make;
	char name[40];
	char serialnumber[40];
});

PACKED(struct vtl_info {
	char name[TL_MAX_NAME_LEN];
	char serialnumber[40];
	uint16_t tl_id;
	uint16_t slots;
	uint8_t type;
	uint8_t ieports;
	uint8_t drives;
	uint8_t disdedupe;
	struct drive_info drive_info[TL_MAX_DRIVES_LEGACY];
	uint16_t dtstart;
	uint16_t ststart;
	uint16_t iestart;
	uint16_t mtstart;
});

#define MAX_ERASE_BLOCKS	16
PACKED(struct raw_partition {
	uint64_t size;
	uint64_t block;
	uint64_t used;
	uint64_t mlookup_block;
	uint64_t erase_blocks[16];
});

#define TAPE_FLAGS_V2		0x1
#define TAPE_FLAGS_V3		0x2
#define TAPE_FLAGS_V4		0x4
#define TAPE_FLAGS_V5		0x8
#define TAPE_FLAGS_ENC		0x10
#define TAPE_FLAGS_TMAP		0x20

PACKED(struct raw_replication_state {
	uint8_t partition_id;
	uint16_t dest_tl_id;
	uint8_t need_sync;
	uint64_t lid_start;
	uint32_t dest_tape_id;
	uint32_t dest_addr;
	uint64_t time;
	uint8_t dest_vcart[TDISK_MAX_NAME_LEN];
	char dest_tape_rid[TL_RID_MAX];
	char sys_rid[TL_RID_MAX];
});

#define MAX_REPLICATION_STATES		8

enum {
	MCHANGER_FLAGS_AVOLTAG,
};

PACKED(struct raw_tape {
	uint16_t csum;
	uint32_t vstatus;
	uint16_t flags;
	uint64_t size;
	uint64_t set_size; /* SET CAPACITY */
	char label[40];
	uint32_t tape_id;
	uint16_t device_type;
	uint16_t group_id;
	uint16_t make;
	uint16_t worm;
	struct vtl_info vtl_info;
	uint64_t last_write1;
	uint64_t last_write;
	uint64_t pad1[1];
	struct raw_partition raw_partitions[MAX_TAPE_PARTITIONS];
	struct raw_replication_state replication_states[MAX_REPLICATION_STATES];
	char sys_rid[TL_RID_MAX];
	struct drive_info_v2 drive_info_v2[TL_MAX_DRIVES - TL_MAX_DRIVES_LEGACY];
	uint64_t data_size[MAX_TAPE_PARTITIONS];
	uint8_t  medium_serial_number[32];
	struct raw_export_state export_states[MAX_TAPE_PARTITIONS];
	char vendor[8];
	char drive_vendor[8];
	char product[16];
	char drive_product[16];
	char revision[8];
	char drive_revision[8];
	char suffix[8];
	char drive_suffix[8];
	char prefix[10];
	char drive_prefix[10];
	uint8_t inquirylen;
	uint8_t drive_inquirylen;
	uint8_t mediatype;
	uint8_t avoltag;
});

#define MAX_VTAPES		60000
#define BDEV_META_RESERVED	262144
#define BDEV_META_OFFSET	BDEV_META_RESERVED
#define BDEV_STATS_OFFSET	(BDEV_META_OFFSET - 32768)
#define BINT_UNIT_SHIFT		27
#define BINT_UNIT_SIZE		(1ULL << BINT_UNIT_SHIFT)
#define BINT_UNIT_MASK		(~(BINT_UNIT_SIZE - 1))
#define BINT_RESERVED_SEGMENTS	4
#define BINT_RESERVED_SIZE	(BINT_UNIT_SIZE * BINT_RESERVED_SEGMENTS)

#define VTAPES_OFFSET		(BDEV_META_OFFSET + LBA_SIZE)
#define DEFAULT_IE_PORTS	0x04
#define ERASE_MAGIC		65

enum {
	MEDIA_VAULTED		= 0x01,
	MEDIA_EXPORT_PENDING		= 0x02,
	MEDIA_REPLICATION_PENDING	= 0x04,
	MEDIA_DELETING			= 0x08,
	MEDIA_SKIP_LOAD			= 0x10,
	MEDIA_FROM_IMPORT		= 0x20,
	MEDIA_ERASE			= 0x40,
};

#define IOCTL_MAX_BUFFER_SIZE			65536

PACKED(struct init_info {
	uint64_t i_prt[0];
	uint64_t t_prt[0];
	char name[256];
	uint8_t init_int;
	uint16_t info_length;
});

struct sense_modify {
	int tl_id;
	int target_id;
	int deletesense;
	uint8_t response_code;
	uint8_t sense_key;
	uint8_t asc;
	uint8_t ascq;
};

PACKED(struct sense_code {
	uint8_t response_code;
	uint8_t sense_key;
	uint8_t asc;
	uint8_t ascq;
});

#ifdef FREEBSD
#define VDEVICE_DUMP_MAX_BUFFER		4096 
#else
#define VDEVICE_DUMP_MAX_BUFFER		8000
#endif
struct vdevicedump {
	uint16_t msg_id;
	uint16_t partition_id;
	uint32_t tl_id;
	uint32_t target_id;
	int buf_length;
	uint8_t buf[VDEVICE_DUMP_MAX_BUFFER];
	char errmsg[100];
};

PACKED(struct raw_mam {
	uint64_t used;
	uint16_t csum;
	uint16_t pad[3];
});

PACKED(struct raw_attribute_v3 {
	uint16_t identifier;
	uint16_t length;
	uint16_t format;
	uint8_t value[0];
});

PACKED(struct log_parameter {
	uint16_t parameter_code;
	uint8_t parameter_flags;
	uint8_t parameter_length;
	uint8_t parameter_value[0];
});

enum {
	VDEVICE_DUMP_SENSE_CODES = 0x01,
	VDEVICE_DUMP_MAM,
	VDEVICE_MAM_ADD_ATTRIBUTE,
	VDEVICE_MAM_DELETE_ATTRIBUTE,
	VDEVICE_DUMP_LOG_PAGE,
	VDEVICE_LOG_PARAM_ADD,
	VDEVICE_LOG_PARAM_DELETE,
	VDEVICE_DUMP_RWCTL,
	VDEVICE_RWCTL_ADD,
	VDEVICE_RWCTL_DELETE,
	VDEVICE_DUMP_REPLICATION_STATE,
};

#define DEFAULT_MT_START_ADDRESS		0
#define DEFAULT_DT_START_ADDRESS		256	
#define DEFAULT_IE_START_ADDRESS		768
#define DEFAULT_ST_START_ADDRESS		1024

#define MAX_STORAGE_SLOTS			60000
#define MAX_IE_PORTS				250

struct rwctl {
	uint64_t lid_start;
	uint32_t num_blocks;
	uint8_t write;
	uint8_t response_code;
	uint8_t sense_key;
	uint8_t asc;
	uint8_t ascq;
};

struct node_config {
	int node_type;
	char node_host[16];
	char controller_host[16];
	char recv_host[16];
	char ha_host[16];
	char ha_bind_host[16];
	uint32_t nodes[64];
	uint32_t node_ipaddr;
	uint32_t controller_ipaddr;
	uint32_t recv_ipaddr;
	uint32_t ha_ipaddr;
	uint32_t ha_bind_ipaddr;
	int sync_status;
	int node_flags;
	int node_role;
	int recv_flags;
	int fence_enabled;
	uint16_t mirror_recv_timeout;
	uint16_t mirror_send_timeout;
	uint16_t mirror_flush_timeout;
	uint16_t mirror_connect_timeout;
};

enum {
	OP_MAP_VTL		= 0x01,
	OP_UNMAP_VTL		= 0x02,
};

struct dlocate_info {
	uint32_t host;
	uint32_t channel;
	uint32_t target;
	uint64_t lun;
	uint32_t libid;
	uint32_t driveid;
	int type;
	int tl_id;
	int make;
	int tl_hostid;
	uint32_t tl_targetid;
};

struct sdevice {
	struct dlocate_info dinfo;
	void *device;
};

enum {
	JOB_STATUS_OK		= 0x0,
	JOB_STATUS_ERROR	= 0x1,
	JOB_STATUS_CANCELED	= 0x2,
	JOB_STATUS_INPROGRESS	= 0x4,
	JOB_STATUS_QUEUED	= 0x5,
	JOB_STATUS_QUEUED2	= 0x6,
	JOB_STATUS_PAUSED	= 0x7,
};

struct iejob_info {
	struct dlocate_info dinfo;
	int isexport;
	int verify;
	int compression;
	int status;
	struct job_stats job_stats;
};

#define TDRIVE_MIN_BLOCK_SIZE		1
#define TDRIVE_MAX_BLOCK_SIZE		16777215

struct hba_info {
	int host;
	int enable;
	int debug;
	int emulate_changer;
	uint64_t wwpn;
};

#define FC_INITIATOR_MODE		0x01
#define FC_TARGET_MODE			0x02
#define FC_DUAL_MODE			(FC_INITIATOR_MODE | FC_TARGET_MODE)
static inline char *
get_hba_mode_str(int mode)
{
	switch (mode) {
	case FC_INITIATOR_MODE:
		return "initiator";
	case FC_TARGET_MODE:
		return "target";
	case FC_DUAL_MODE:
		return "initiator|target";
	default:
		return "unknown";
	}
}

static inline char *
get_voltag_suffix(int voltype, int worm)
{
	switch(voltype) {
	case VOL_TYPE_LTO_1:
		if (!worm)
			return "L1";
		else
			return "LR";
	case VOL_TYPE_LTO_2:
		if (!worm)
			return "L2";
		else
			return "LS";
	case VOL_TYPE_LTO_3:
		if (!worm)
			return "L3";
		else
			return "LT";
	case VOL_TYPE_LTO_4:
		if (!worm)
			return "L4";
		else
			return "LU";
	case VOL_TYPE_LTO_5:
		if (!worm)
			return "L5";
		else
			return "LV";
	case VOL_TYPE_LTO_6:
		if (!worm)
			return "L6";
		else
			return "LW";
	case VOL_TYPE_LTO_7:
		if (!worm)
			return "L7";
		else
			return "LX";
	case VOL_TYPE_LTO_8:
		if (!worm)
			return "L8";
		else
			return "LY";
	case VOL_TYPE_LTO_9:
		if (!worm)
			return "L9";
		else
			return "LZ";
	case VOL_TYPE_DLT_4:
	case VOL_TYPE_VSTAPE:
		return "";
	case VOL_TYPE_SDLT_1:
	case VOL_TYPE_SDLT_2:
	case VOL_TYPE_SDLT_3:
		return "S";
	case VOL_TYPE_T9840_A:
	case VOL_TYPE_T9840_B:
	case VOL_TYPE_T9840_C:
	case VOL_TYPE_T9840_D:
		return "R";
	case VOL_TYPE_T9940_A:
	case VOL_TYPE_T9940_B:
		return "P";
	case VOL_TYPE_T10000_A:
	case VOL_TYPE_T10000_B:
		return "T1";
	case VOL_TYPE_T10000_C:
	case VOL_TYPE_T10000_D:
		return "T2";
	}
	return "";
}

enum {
	DISK_OP_ADD		= 0x01,
	DISK_OP_DELETE,
	DISK_OP_ENABLE,
	DISK_OP_DISABLE,
	DISK_OP_MIGRATE,
	DISK_OP_MIGRATE_CANCEL,
	DISK_OP_REPLACE,
	DISK_OP_MIGRATE_ERROR,
	DISK_OP_REPLACE_ERROR,
	DISK_OP_MIGRATE_CANCELED,
	DISK_OP_DONE_REPLACE_POST,
};

struct ldev_info {
	uint32_t tid;
	uint32_t lun_id;
	uint32_t iscsi_tid;
	uint32_t vhba_id;
};

enum {
	NODE_STATUS_OK			= SCSI_STATUS_OK,
	NODE_STATUS_SCSI_SENSE		= SCSI_STATUS_CHECK_COND,
	NODE_STATUS_BUSY		= SCSI_STATUS_BUSY,
	NODE_STATUS_RESERV_CONFLICT	= SCSI_STATUS_RESERV_CONFLICT,

	/* Custom codes */
	NODE_STATUS_TARGET_NOT_FOUND	= 0xE0,
	NODE_STATUS_INVALID_MSG		= 0xE1,
	NODE_STATUS_UNREGISTERED_NODE	= 0xE2,
	NODE_STATUS_MEM_ALLOC_FAILURE	= 0xE3,
	NODE_STATUS_ERROR		= 0xE4,
	NODE_STATUS_AMAP_NOT_FOUND	= 0xE5,
	NODE_STATUS_LID_MISMATCH	= 0xE6,
	NODE_STATUS_INITIALIZING	= 0xE7,
	NODE_STATUS_NEED_RESYNC		= 0xE8,
	NODE_STATUS_MIRROR_CONFIGURED	= 0xE9,
	NODE_STATUS_INVALID_MIRROR_CONFIGURATION	= 0xEA,
	NODE_STATUS_SKIP_LOCAL_WRITE	= 0xEB,
	NODE_STATUS_IS_MASTER		= 0xEC,
	NODE_STATUS_DO_LOCAL_READ	= 0xED,
	NODE_STATUS_OK_V4		= 0xEF,
};

struct core_node_msg {
	uint32_t csum;
	uint32_t dxfer_len;
	uint8_t  msg_cmd;
	uint8_t  msg_status;
	uint16_t pad;
	uint32_t pad1;
	uint64_t msg_id;
	uint8_t  data[0];
};

static inline void
calc_sector_bits(uint32_t sector_size, uint32_t *sector_shift)
{
	uint32_t shift;

	shift = 0;
	while ((sector_size >>= 1))
	{
		shift++;
	}
	*sector_shift = shift;
	return;
}

struct page_spec {
	unsigned long addr;
};

struct pgdata_spec {
	unsigned long addr;
	uint16_t pg_len;
	uint16_t pg_offset;
	uint32_t pad;
};

#define IOCTL_PGADDR_MAX	256
struct bio_spec {
	unsigned long kfd;
	uint8_t rw;
	uint8_t file;
	uint16_t page_count;
	uint32_t length;
	uint64_t b_start;
	unsigned long bio_addr;
	struct pgdata_spec *pglist;
};

struct comp_spec {
	uint16_t cpage_count;
	uint16_t upage_count;
	uint16_t compress;
	uint16_t pad;
	uint32_t comp_size;
	uint32_t uncomp_size;
	struct page_spec *cpages;
};

struct file_spec {
	unsigned long kfd;
	unsigned long length;
	int error;
	int isnew;
	char name[4096];
};

struct bdev_sync {
	unsigned long kfd;
};

struct tdev_spec {
	unsigned long kfd;
	int error;
	uint32_t host;
	uint32_t channel;
	uint32_t target;
	uint64_t lun;
	char name[1024];
};

struct bdev_spec {
	unsigned long kfd;
	int error;
	int pad;
	uint32_t sector_shift;
	uint32_t sector_size;
	uint64_t size;
	char name[1024];
};

struct ctio_mmap_setup {
	unsigned long addr;
	uint32_t lun_id;
	uint32_t pad;
};

struct bio_mmap_setup {
	unsigned long addr;
};

struct ctio_wakeup {
	uint32_t lun_id;
	uint32_t pad;
};

static inline uint32_t
csum32(uint8_t *buf, int len)
{
        int i;
        uint32_t csum = 0;

        for (i = 0; (i + 4) <= len ; i+=4)
        {
                uint32_t val = *((uint32_t *)(buf+i));
                csum += val;
        }
	while (i < len)
		csum += buf[i++];
        return ~(csum);
}

enum {
	COMP_ALG_LZ4	= 0x01,
};

#define COMP_ALG_SHIFT	28	
#define COMP_ALG_MASK	((1 << COMP_ALG_SHIFT) - 1)
#define SET_COMP_SIZE(ptr, csize, alg)	(*((uint32_t *)(ptr)) = (csize | (alg << COMP_ALG_SHIFT)))

struct tapecmd {
	uint8_t cdb[16];
	uint8_t *data;
	uint32_t dxfer_out;
	uint32_t dxfer_in;
	uint32_t scsi_status;
	uint32_t device_tid;
	uint32_t device_lunid;
	uint8_t sense[32];
	uint32_t sense_len;
};

struct tapebuf {
	uint8_t *data;
	uint32_t dxfer_len;
	uint32_t csum;
	uint32_t scsi_status;
	uint32_t device_tid;
	uint32_t device_lunid;
	uint8_t sense[32];
	uint32_t sense_len;
};

static inline uint64_t
align_size(uint64_t size, int boundary)
{
	uint64_t adjust;

	adjust = (size + (boundary - 1)) & -(boundary);
	return (adjust);
}

#define BIO_PAGE_INCREMENT	128

#endif /* COMMONDEFS_H_ */
